let board = null;
let game = new Chess();
let timer = null;
let timeLeft = 20;
let isPlayerTurn = true;
let playerBlunders = 0;
let playerMistakes = 0;
let computerBlunders = 0;
let computerMistakes = 0;
let gameStarted = false;
let playerColor = 'white';
let isFirstVisit = true;

// Initialize the board
function initializeBoard() {
    const pieceTheme = 'img/chesspieces/wikipedia/{piece}.png';
    board = Chessboard('board', {
        position: 'start',
        draggable: true,
        onDragStart: onDragStart,
        onDrop: onDrop,
        onSnapEnd: onSnapEnd,
        pieceTheme: pieceTheme
    });
    
    // Only start timer if game has been started
    if (!isFirstVisit && gameStarted) {
        startTimer();
    }
}

// Handle piece drag start
function onDragStart(source, piece) {
    if (game.game_over()) return false;
    if (!isPlayerTurn) return false;
    if ((game.turn() === 'w' && piece.search(/^b/) !== -1) ||
        (game.turn() === 'b' && piece.search(/^w/) !== -1)) {
        return false;
    }
}

// Handle piece drop
function onDrop(source, target) {
    const move = game.move({
        from: source,
        to: target,
        promotion: 'q'
    });

    if (move === null) return 'snapback';

    // Evaluate the move
    evaluateMove(move, true);
    
    // Check for game end
    if (game.game_over()) {
        handleGameEnd();
        return;
    }
    
    // Reset timer for computer's turn
    resetTimer();
    isPlayerTurn = false;

    // Make computer move after a short delay
    setTimeout(makeComputerMove, 500);
}

// Handle game end
function handleGameEnd() {
    clearInterval(timer);
    const banner = document.getElementById('gameEndBanner');
    
    if (game.in_checkmate()) {
        banner.textContent = 'Checkmate!';
    } else if (game.in_stalemate()) {
        banner.textContent = 'Stalemate!';
    } else if (game.in_draw()) {
        banner.textContent = 'Draw!';
    }
    
    banner.style.display = 'block';
    setTimeout(() => {
        banner.style.display = 'none';
    }, 3000);
}

// Make computer move
function makeComputerMove() {
    if (game.game_over()) {
        handleGameEnd();
        return;
    }

    const elo = parseInt(document.getElementById('elo').value);
    const move = getComputerMove(elo);
    
    if (move) {
        game.move(move);
        board.position(game.fen());
        
        // Evaluate the move
        evaluateMove(move, false);
        
        // Check for game end
        if (game.game_over()) {
            handleGameEnd();
            return;
        }
        
        // Reset timer for player's turn
        resetTimer();
        isPlayerTurn = true;
    }
}

// Get computer move based on ELO
function getComputerMove(elo) {
    const moves = game.moves({ verbose: true });
    if (moves.length === 0) return null;

    // Normalize ELO to a value between 0 and 1
    // 500 ELO = 0.0 (random moves)
    // 1500 ELO = 1.0 (best moves)
    const normalizedElo = Math.min(1.0, Math.max(0.0, (elo - 500) / 1000));
    
    // Evaluate each move with a sophisticated heuristic
    const evaluatedMoves = moves.map(move => {
        // Make the move
        game.move(move);
        
        // Get evaluation score
        let score = evaluatePosition();
        
        // Apply ELO-based depth for move consideration
        // Higher ELO = look deeper into the position
        const depth = Math.floor(normalizedElo * 2); // 0 to 2 moves ahead
        if (depth > 0) {
            const counterMoves = game.moves({ verbose: true });
            if (counterMoves.length > 0) {
                // Look at opponent's best responses
                const counterEvaluations = counterMoves.map(counterMove => {
                    game.move(counterMove);
                    const counterScore = evaluatePosition();
                    game.undo();
                    return counterScore;
                });
                
                // Consider the worst counter-move (best for opponent)
                const worstCounterScore = Math.min(...counterEvaluations);
                score = Math.min(score, worstCounterScore);
            }
        }
        
        // Undo the move
        game.undo();
        
        return { move, score };
    });
    
    // Sort moves by score (descending)
    evaluatedMoves.sort((a, b) => b.score - a.score);
    
    // Select a move based on ELO
    // Higher ELO = more likely to select a better move
    const moveIndex = Math.floor(Math.random() * Math.max(1, Math.floor(evaluatedMoves.length * (1 - normalizedElo * 0.95))));
    return evaluatedMoves[moveIndex].move;
}

// Sophisticated position evaluation
function evaluatePosition() {
    let score = 0;
    
    // Material values (adjusted for ELO)
    const materialValues = {
        'p': 1,    // pawn
        'n': 3,    // knight
        'b': 3.2,  // bishop
        'r': 5,    // rook
        'q': 9,    // queen
        'k': 0     // king
    };
    
    // Piece position bonuses (center control, development, etc.)
    const positionBonuses = {
        'p': [ // Pawn position values
            0,  0,  0,  0,  0,  0,  0,  0,
            0.5, 0.5, 0.5, 0.5, 0.5, 0.5, 0.5, 0.5,
            0.1, 0.1, 0.2, 0.3, 0.3, 0.2, 0.1, 0.1,
            0.05, 0.05, 0.1, 0.25, 0.25, 0.1, 0.05, 0.05,
            0,  0,  0,  0.2, 0.2,  0,  0,  0,
            0.05, -0.05, -0.1, 0, 0, -0.1, -0.05, 0.05,
            0.1, 0.1, 0.2, -0.2, -0.2, 0.2, 0.1, 0.1,
            0,  0,  0,  0,  0,  0,  0,  0
        ],
        'n': [ // Knight position values
            -0.5, -0.4, -0.3, -0.3, -0.3, -0.3, -0.4, -0.5,
            -0.4, -0.2,  0,  0,  0,  0, -0.2, -0.4,
            -0.3,  0, 0.1, 0.15, 0.15, 0.1,  0, -0.3,
            -0.3, 0.05, 0.15, 0.2, 0.2, 0.15, 0.05, -0.3,
            -0.3,  0, 0.15, 0.2, 0.2, 0.15,  0, -0.3,
            -0.3, 0.05, 0.1, 0.15, 0.15, 0.1, 0.05, -0.3,
            -0.4, -0.2,  0, 0.05, 0.05,  0, -0.2, -0.4,
            -0.5, -0.4, -0.3, -0.3, -0.3, -0.3, -0.4, -0.5
        ],
        'b': [ // Bishop position values
            -0.2, -0.1, -0.1, -0.1, -0.1, -0.1, -0.1, -0.2,
            -0.1,  0,  0,  0,  0,  0,  0, -0.1,
            -0.1,  0, 0.05, 0.1, 0.1, 0.05,  0, -0.1,
            -0.1, 0.05, 0.05, 0.1, 0.1, 0.05, 0.05, -0.1,
            -0.1,  0, 0.1, 0.1, 0.1, 0.1,  0, -0.1,
            -0.1, 0.1, 0.1, 0.1, 0.1, 0.1, 0.1, -0.1,
            -0.1, 0.05,  0,  0,  0,  0, 0.05, -0.1,
            -0.2, -0.1, -0.1, -0.1, -0.1, -0.1, -0.1, -0.2
        ],
        'r': [ // Rook position values
            0,  0,  0,  0,  0,  0,  0,  0,
            0.05, 0.1, 0.1, 0.1, 0.1, 0.1, 0.1, 0.05,
            -0.05,  0,  0,  0,  0,  0,  0, -0.05,
            -0.05,  0,  0,  0,  0,  0,  0, -0.05,
            -0.05,  0,  0,  0,  0,  0,  0, -0.05,
            -0.05,  0,  0,  0,  0,  0,  0, -0.05,
            -0.05,  0,  0,  0,  0,  0,  0, -0.05,
            0,  0,  0, 0.05, 0.05,  0,  0,  0
        ],
        'q': [ // Queen position values
            -0.2, -0.1, -0.1, -0.05, -0.05, -0.1, -0.1, -0.2,
            -0.1,  0,  0,  0,  0,  0,  0, -0.1,
            -0.1,  0, 0.05, 0.05, 0.05, 0.05,  0, -0.1,
            -0.05,  0, 0.05, 0.05, 0.05, 0.05,  0, -0.05,
            0,  0, 0.05, 0.05, 0.05, 0.05,  0, -0.05,
            -0.1, 0.05, 0.05, 0.05, 0.05, 0.05,  0, -0.1,
            -0.1,  0,  0,  0,  0,  0,  0, -0.1,
            -0.2, -0.1, -0.1, -0.05, -0.05, -0.1, -0.1, -0.2
        ],
        'k': [ // King position values (middlegame)
            -0.3, -0.4, -0.4, -0.5, -0.5, -0.4, -0.4, -0.3,
            -0.3, -0.4, -0.4, -0.5, -0.5, -0.4, -0.4, -0.3,
            -0.3, -0.4, -0.4, -0.5, -0.5, -0.4, -0.4, -0.3,
            -0.3, -0.4, -0.4, -0.5, -0.5, -0.4, -0.4, -0.3,
            -0.2, -0.3, -0.3, -0.4, -0.4, -0.3, -0.3, -0.2,
            -0.1, -0.2, -0.2, -0.2, -0.2, -0.2, -0.2, -0.1,
            0.2, 0.2,  0,  0,  0,  0, 0.2, 0.2,
            0.2, 0.3, 0.1,  0,  0, 0.1, 0.3, 0.2
        ]
    };
    
    // Get current ELO for evaluation scaling
    const elo = parseInt(document.getElementById('elo').value);
    const normalizedElo = Math.min(1.0, Math.max(0.0, (elo - 500) / 1000));
    
    // Evaluate the position
    for (let i = 0; i < 8; i++) {
        for (let j = 0; j < 8; j++) {
            const square = game.board()[i][j];
            if (square) {
                const piece = square.type.toLowerCase();
                const value = materialValues[piece];
                const position = i * 8 + j;
                
                // Add material value
                if (square.color === 'b') {
                    score += value;
                    // Add position bonus (scaled by ELO)
                    score += positionBonuses[piece][position] * normalizedElo;
                } else {
                    score -= value;
                    // Subtract position bonus (flip board for black pieces)
                    score -= positionBonuses[piece][63 - position] * normalizedElo;
                }
            }
        }
    }
    
    // Evaluate mobility (scaled by ELO)
    const mobilityScore = evaluateMobility() * normalizedElo;
    score += mobilityScore;
    
    // Evaluate king safety (scaled by ELO)
    const kingSafetyScore = evaluateKingSafety() * normalizedElo;
    score += kingSafetyScore;
    
    // Add randomness based on ELO
    // Lower ELO = more randomness
    const randomness = (Math.random() * 2 - 1) * (1 - normalizedElo) * 2.0;
    score += randomness;
    
    return score;
}

// Evaluate piece mobility
function evaluateMobility() {
    let score = 0;
    const moves = game.moves({ verbose: true });
    
    // Count moves for each piece
    for (const move of moves) {
        const piece = game.get(move.from).type.toLowerCase();
        const value = {
            'p': 0.1,  // Pawns
            'n': 0.2,  // Knights
            'b': 0.2,  // Bishops
            'r': 0.3,  // Rooks
            'q': 0.4   // Queens
        }[piece] || 0;
        
        if (game.turn() === 'b') {
            score += value;
        } else {
            score -= value;
        }
    }
    
    return score;
}

// Evaluate king safety
function evaluateKingSafety() {
    let score = 0;
    
    // Find kings
    let whiteKingSquare = null;
    let blackKingSquare = null;
    
    for (let i = 0; i < 8; i++) {
        for (let j = 0; j < 8; j++) {
            const square = game.board()[i][j];
            if (square && square.type.toLowerCase() === 'k') {
                if (square.color === 'w') {
                    whiteKingSquare = { i, j };
                } else {
                    blackKingSquare = { i, j };
                }
            }
        }
    }
    
    // Evaluate pawn shield
    if (whiteKingSquare) {
        score -= evaluatePawnShield(whiteKingSquare, 'w');
    }
    if (blackKingSquare) {
        score += evaluatePawnShield(blackKingSquare, 'b');
    }
    
    return score;
}

// Evaluate pawn shield around king
function evaluatePawnShield(kingSquare, color) {
    let score = 0;
    const direction = color === 'w' ? -1 : 1;
    
    // Check for pawns in front of king
    for (let i = -1; i <= 1; i++) {
        const file = kingSquare.j + i;
        if (file >= 0 && file < 8) {
            const rank = kingSquare.i + direction;
            if (rank >= 0 && rank < 8) {
                const square = game.board()[rank][file];
                if (square && square.type.toLowerCase() === 'p' && square.color === color) {
                    score += 0.2; // Bonus for each pawn shield
                }
            }
        }
    }
    
    return score;
}

// Evaluate a move for blunders and mistakes
function evaluateMove(move, isPlayer) {
    // Store the current position evaluation
    const beforeScore = evaluatePosition();
    
    // Make the move
    game.move(move);
    
    // Get the evaluation after the move
    const afterScore = evaluatePosition();
    
    // Calculate the evaluation difference
    const evalDiff = afterScore - beforeScore;
    
    // Define thresholds for blunders and mistakes
    // These values are based on material value (1 pawn = 1.0)
    const BLUNDER_THRESHOLD = -2.0;  // Losing more than 2 pawns worth of material/position
    const MISTAKE_THRESHOLD = -0.5;  // Losing more than half a pawn worth of material/position
    
    // Adjust thresholds based on game phase
    const gamePhase = getGamePhase();
    const adjustedBlunderThreshold = BLUNDER_THRESHOLD * (gamePhase === 'opening' ? 1.5 : 1.0);
    const adjustedMistakeThreshold = MISTAKE_THRESHOLD * (gamePhase === 'opening' ? 1.5 : 1.0);
    
    // Determine if it's a blunder or mistake
    if (evalDiff < adjustedBlunderThreshold) {
        if (isPlayer) {
            playerBlunders++;
            document.getElementById('playerBlunders').textContent = playerBlunders;
        } else {
            computerBlunders++;
            document.getElementById('computerBlunders').textContent = computerBlunders;
        }
    } else if (evalDiff < adjustedMistakeThreshold) {
        if (isPlayer) {
            playerMistakes++;
            document.getElementById('playerMistakes').textContent = playerMistakes;
        } else {
            computerMistakes++;
            document.getElementById('computerMistakes').textContent = computerBlunders;
        }
    }
    
    // Undo the move to restore the position
    game.undo();
}

// Get game phase
function getGamePhase() {
    const pieceCount = game.board().flat().filter(square => square !== null).length;
    if (pieceCount > 30) return 'opening';
    if (pieceCount > 20) return 'middlegame';
    return 'endgame';
}

// Timer functions
function startTimer() {
    if (!gameStarted) return;
    
    timeLeft = parseInt(document.getElementById('timer').value);
    updateTimerDisplay();
    
    if (timer) clearInterval(timer);
    
    timer = setInterval(() => {
        timeLeft--;
        updateTimerDisplay();
        
        if (timeLeft <= 0) {
            clearInterval(timer);
            if (isPlayerTurn) {
                alert('Time\'s up! You lost on time.');
                resetGame();
            }
        }
    }, 1000);
}

function resetTimer() {
    if (!gameStarted) return;
    
    timeLeft = parseInt(document.getElementById('timer').value);
    updateTimerDisplay();
}

function updateTimerDisplay() {
    document.getElementById('timeRemaining').textContent = timeLeft;
}

// Reset game
function resetGame() {
    game = new Chess();
    board.position('start');
    playerBlunders = 0;
    playerMistakes = 0;
    computerBlunders = 0;
    computerMistakes = 0;
    document.getElementById('playerBlunders').textContent = '0';
    document.getElementById('playerMistakes').textContent = '0';
    document.getElementById('computerBlunders').textContent = '0';
    document.getElementById('computerMistakes').textContent = '0';
    
    // Get player color from radio buttons
    playerColor = document.querySelector('input[name="color"]:checked').value;
    
    // Set initial turn based on player color
    isPlayerTurn = (playerColor === 'white');
    
    // Flip the board if player is black
    if (playerColor === 'black') {
        board.flip();
    } else {
        board.orientation('white');
    }
    
    // Start the game and timer
    gameStarted = true;
    isFirstVisit = false;
    resetTimer();
    startTimer();
    
    // If player is black, make computer move first
    if (playerColor === 'black') {
        setTimeout(makeComputerMove, 500);
    }
}

// Event listeners
document.addEventListener('DOMContentLoaded', function() {
    // Initialize the board
    initializeBoard();
    
    // Add event listener for the start button
    document.getElementById('startBtn').addEventListener('click', resetGame);
    
    // Add event listeners for color selection
    document.getElementById('colorWhite').addEventListener('change', function() {
        if (this.checked) {
            playerColor = 'white';
            if (gameStarted) {
                board.orientation('white');
                isPlayerTurn = true;
            }
        }
    });
    
    document.getElementById('colorBlack').addEventListener('change', function() {
        if (this.checked) {
            playerColor = 'black';
            if (gameStarted) {
                board.flip();
                isPlayerTurn = false;
                setTimeout(makeComputerMove, 500);
            }
        }
    });
    
    // Initialize timer display
    updateTimerDisplay();
}); 